/* (C) 2022 YiRing, Inc. */
package com.yiring.app.design.strategy.rule;

import com.yiring.app.constant.alarm.RelevanceParamEnum;
import com.yiring.app.design.strategy.IParamInitStrategy;
import com.yiring.app.design.strategy.IScanAlarmStrategy;
import com.yiring.app.domain.alarm.AlarmType;
import com.yiring.app.domain.location.LocationFence;
import com.yiring.app.domain.location.LocationFenceAlarm;
import com.yiring.app.domain.location.LocationTag;
import com.yiring.app.param.location.fence.LocationFenceJobParam;
import com.yiring.app.param.location.rule.LocationFenceRuleAddParam;
import com.yiring.app.param.location.rule.RuleParam;
import com.yiring.app.vo.location.fence.LocationFenceJobVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import org.springframework.stereotype.Component;

/**
 * 静止时长相关策略
 * @author tml
 * @version 1.0
 * @date 2022/4/29 9:32
 */
@Component
public class StaticDurationStrategy implements IParamInitStrategy<LocationFenceRuleAddParam>, IScanAlarmStrategy {

    public StaticDurationStrategy() {
        Integer relevanceParam = RelevanceParamEnum.STATIC_DURATION.getCode();
        LocationFenceRuleContext.register(relevanceParam, this);
        ScanAlarmContext.register(relevanceParam, this);
    }

    @Override
    public Result<String> paramInit(LocationFenceRuleAddParam param) {
        RuleParam ruleParam = param.getRuleParam();
        if (ruleParam.getDuration() == null || ruleParam.getDuration() < 0) {
            return Result.no(Status.EXPECTATION_FAILED, "静止时长异常");
        }
        ruleParam.setDistance(-1);
        ruleParam.setTimeAndUser(new ArrayList<>());
        ruleParam.setTimeAndNumber(new ArrayList<>());
        return Result.ok();
    }

    @Override
    public LocationFenceJobVo scanAlarm(LocationFenceJobParam param) {
        RuleParam rule = param.getRule();
        Set<LocationTag> tagSet = param.getTagSet();
        Long mapId = param.getMapId();
        List<LocationFenceAlarm> oldAlarms = param.getOldAlarms();
        Long fenceId = param.getFenceId();
        Long alarmTypeId = param.getAlarmTypeId();
        LocationFenceJobVo jobVo = new LocationFenceJobVo();

        //筛选出围栏中静止的，并且静止时长超过配置规则时长的标签
        LocalDateTime time = LocalDateTime.now().minusMinutes(rule.getDuration());
        List<LocationFenceAlarm> newAlarms = tagSet
            .stream()
            .filter(LocationTag::getSilent)
            .filter(e -> e.getTime().isBefore(time))
            .map(e -> {
                LocationFence fence = LocationFence.builder().id(fenceId).name(param.getFenceName()).build();
                AlarmType alarmType = AlarmType.builder().id(alarmTypeId).build();
                return LocationFenceAlarm
                    .builder()
                    .fence(fence)
                    .point(e.getPoint())
                    .areaId(mapId)
                    .user(e.getUser())
                    .tag(e)
                    .startTime(LocalDateTime.now())
                    .type(alarmType)
                    .status(LocationFenceAlarm.Status.ING)
                    .build();
            })
            .collect(Collectors.toList());
        //取两个集合没有相交的部分，newAlarms不重合的部分代表需要添加的报警，oldAlarms不重合的部分代表需要结束的报警
        IScanAlarmStrategy.notIntersect(newAlarms, oldAlarms, jobVo);
        return jobVo;
    }
}
