/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.location.fence.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.app.domain.location.LocationFence;
import com.yiring.app.domain.location.LocationFenceAlarm;
import com.yiring.app.domain.location.LocationFenceAlarmRepository;
import com.yiring.app.domain.location.LocationFenceRepository;
import com.yiring.app.excel.location.fence.FenceAlarmExcel;
import com.yiring.app.param.location.fence.FenceAlarmConditionParam;
import com.yiring.app.service.location.fence.FenceAlarmService;
import com.yiring.app.util.MapUtil;
import com.yiring.app.vo.IdNameVo;
import com.yiring.app.vo.location.fence.FenceAlarmVo;
import com.yiring.auth.domain.user.User;
import com.yiring.common.core.Result;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author tml
 * @version 1.0
 * @date 2022/5/17 16:27
 */
@Slf4j
@Service
@Transactional(rollbackFor = Exception.class)
public class FenceAlarmServiceImpl implements FenceAlarmService {

    @Resource
    private LocationFenceAlarmRepository locationFenceAlarmRepository;

    @Resource
    private LocationFenceRepository locationFenceRepository;

    @Resource
    SimpMessagingTemplate simpMessagingTemplate;

    @Resource
    private MapUtil mapUtil;

    @Override
    public Result<PageVo<FenceAlarmVo>> findList(FenceAlarmConditionParam param, PageParam pageParam) {
        Pageable pageable = PageParam.toPageable(pageParam);
        Specification<LocationFenceAlarm> specification = getSpecification(param);
        Page<LocationFenceAlarm> page = locationFenceAlarmRepository.findAll(specification, pageable);
        List<FenceAlarmVo> voList = page
            .get()
            .map(e -> {
                FenceAlarmVo vo = FenceAlarmVo.transform(e);
                vo.setMapName(mapUtil.getMapName(e.getAreaId().intValue()));
                return vo;
            })
            .collect(Collectors.toList());
        PageVo<FenceAlarmVo> pageVo = PageVo.build(voList, voList.size());
        return Result.ok(pageVo);
    }

    @Override
    public Result<PageVo<IdNameVo>> findFenceList() {
        List<LocationFence> list = locationFenceRepository.findAll();
        List<IdNameVo> voList = list
            .stream()
            .map(e -> new IdNameVo(e.getId(), e.getName()))
            .collect(Collectors.toList());
        PageVo<IdNameVo> pageVo = PageVo.build(voList, voList.size());
        return Result.ok(pageVo);
    }

    @Override
    public void export(FenceAlarmConditionParam param, HttpServletResponse response) {
        Specification<LocationFenceAlarm> specification = getSpecification(param);
        List<LocationFenceAlarm> list = locationFenceAlarmRepository.findAll(specification);
        List<FenceAlarmExcel> excelList = list
            .stream()
            .map(e -> {
                FenceAlarmExcel excel = FenceAlarmExcel.transform(e);
                excel.setMapName(mapUtil.getMapName(e.getAreaId().intValue()));
                return excel;
            })
            .collect(Collectors.toList());
        try (
            DefaultStreamExcelBuilder<FenceAlarmExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(FenceAlarmExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(excelList);

            String fileName = URLEncoder.encode("围栏报警数据列表.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("报警类型列表导出失败: " + e.getMessage());
        }
    }

    @Async
    @Override
    public void alarmPushWeb(List<LocationFenceAlarm> newAlarm) {
        JSONArray jsonArray = new JSONArray();
        for (LocationFenceAlarm item : newAlarm) {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("fence", item.getFence().getName());
            jsonObject.put("map", mapUtil.getMapName(item.getAreaId().intValue()));
            jsonObject.put("user", item.getUser().getRealName());
            jsonObject.put("tagId", item.getTag().getId());
            jsonObject.put("startTime", item.getStartTime());
            jsonObject.put("AlarmType", item.getType().getName());
            jsonObject.put("status", item.getStatus().text());
            jsonArray.add(jsonObject);
        }
        simpMessagingTemplate.convertAndSend("/topic/position", jsonArray.toJSONString());
    }

    private Specification<LocationFenceAlarm> getSpecification(FenceAlarmConditionParam param) {
        return (root, query, criteriaBuilder) -> {
            ArrayList<Predicate> list = new ArrayList<>();
            if (param.getMapId() != null) {
                list.add(criteriaBuilder.equal(root.get(LocationFenceAlarm.Fields.areaId), param.getMapId()));
            }
            if (param.getFenceId() != null) {
                LocationFence fence = LocationFence.builder().id(param.getFenceId()).build();
                list.add(criteriaBuilder.equal(root.get(LocationFenceAlarm.Fields.fence), fence));
            }
            if (param.getRealName() != null) {
                Join<Object, Object> join = root.join(LocationFenceAlarm.Fields.user, JoinType.LEFT);
                list.add(criteriaBuilder.like(join.get(User.Fields.realName), "%" + param.getRealName() + "%"));
            }
            if (param.getStatus() != null) {
                list.add(
                    criteriaBuilder.equal(
                        root.get(LocationFenceAlarm.Fields.status),
                        LocationFenceAlarm.Status.get(param.getStatus())
                    )
                );
            }
            if (param.getBeginTime() != null) {
                list.add(
                    criteriaBuilder.greaterThanOrEqualTo(
                        root.get(LocationFenceAlarm.Fields.startTime),
                        param.getBeginTime()
                    )
                );
            }
            if (param.getEndTime() != null) {
                list.add(
                    criteriaBuilder.lessThanOrEqualTo(root.get(LocationFenceAlarm.Fields.startTime), param.getEndTime())
                );
            }
            Predicate[] array = list.toArray(new Predicate[0]);
            return criteriaBuilder.and(array);
        };
    }
}
